/**
 *
 * Tagcloud module for Magnolia CMS (http://www.openmindlab.com/lab/products/mgnltagcloud.html)
 * Copyright(C) 2010-2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnltagcloud.util;

import info.magnolia.cms.util.FactoryUtil;
import info.magnolia.cms.util.FactoryUtil.InstanceFactory;
import info.magnolia.context.MgnlContext;

import java.lang.reflect.Method;

import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.query.InvalidQueryException;

import org.apache.jackrabbit.core.RepositoryImpl;
import org.apache.jackrabbit.core.SearchManager;
import org.apache.jackrabbit.core.SessionImpl;
import org.apache.jackrabbit.core.query.lucene.LuceneQueryBuilder;
import org.apache.jackrabbit.core.query.lucene.SearchIndex;
import org.apache.jackrabbit.spi.commons.query.QueryNodeFactory;
import org.apache.jackrabbit.spi.commons.query.QueryParser;
import org.apache.jackrabbit.spi.commons.query.QueryRootNode;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Query;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Utility class to get internals on jackrabbit
 * @author molaschi
 * @version $Id: JackrabbitUtil.java 3184 2010-12-14 20:47:38Z fgiust $
 */
public class JackrabbitUtil implements InstanceFactory
{

    private static final String LANG_SQL = "sql";

    /**
     * Logger.
     */
    private Logger log = LoggerFactory.getLogger(JackrabbitUtil.class);

    private Method getSearchManager;

    private Method getQueryNodeFactory;

    /**
     * {@inheritDoc}
     */
    public Object newInstance()
    {
        JackrabbitUtil util = new JackrabbitUtil();
        try
        {
            util.getSearchManager = RepositoryImpl.class.getDeclaredMethod("getSearchManager", String.class);
            util.getQueryNodeFactory = SearchIndex.class.getDeclaredMethod("getQueryNodeFactory");

            util.getSearchManager.setAccessible(true);
            util.getQueryNodeFactory.setAccessible(true);
        }
        catch (SecurityException e)
        {
            log.error("Exception getting SearchManager", e);
        }
        catch (NoSuchMethodException e)
        {
            log.error("Exception getting SearchManager", e);
        }
        return util;
    }

    /**
     * Get jcr session
     * @param repository repository
     * @return jcr session
     */
    public static Session getSession(String repository)
    {
        return MgnlContext.getSystemContext().getHierarchyManager(repository).getWorkspace().getSession();
    }

    /**
     * Get jackrabbit SearchIndex on a repository
     * @param repository repository
     * @param session jcr session
     * @return {@link SearchIndex}
     * @throws RepositoryException exception getting searchindex
     */
    public static SearchIndex getSearchIndex(String repository, Session session) throws RepositoryException
    {
        RepositoryImpl repImpl = (RepositoryImpl) session.getRepository();
        JackrabbitUtil util = (JackrabbitUtil) FactoryUtil.getSingleton(JackrabbitUtil.class);
        try
        {
            SearchManager searchManager = (SearchManager) util.getSearchManager.invoke(repImpl, repository);
            return (SearchIndex) searchManager.getQueryHandler();
        }
        catch (Throwable e)
        {
            util.log.error("Error retrieving SearchIndex", e);
        }
        return null;
    }

    /**
     * Get lucene query to filter faceted search on a path
     * @param path path to filter on
     * @param session jcr session
     * @param si jackr search index
     * @return lucene query
     */
    public static Query getQuery(String path, Session session, SearchIndex si)
    {
        JackrabbitUtil util = (JackrabbitUtil) FactoryUtil.getSingleton(JackrabbitUtil.class);
        QueryNodeFactory factory;
        try
        {
            factory = (QueryNodeFactory) util.getQueryNodeFactory.invoke(si);
        }
        catch (Throwable e)
        {
            util.log.error("Error qetting query node factory", e);
            return null;
        }
        try
        {
            QueryRootNode qrn = QueryParser.parse(
                "SELECT * FROM nt:base WHERE jcr:path LIKE '" + path + "/%'",
                LANG_SQL,
                (SessionImpl) session,
                factory);
            Query queryChild = LuceneQueryBuilder.createQuery(qrn, (SessionImpl) session, si
                .getContext()
                .getItemStateManager(), si.getNamespaceMappings(), si.getTextAnalyzer(), si
                .getContext()
                .getPropertyTypeRegistry(), si.getSynonymProvider(), si.getIndexFormatVersion());
            QueryRootNode qrnSelf = QueryParser.parse(
                "SELECT * FROM nt:base WHERE jcr:path = '" + path + "'",
                LANG_SQL,
                (SessionImpl) session,
                factory);
            Query querySelf = LuceneQueryBuilder.createQuery(qrnSelf, (SessionImpl) session, si
                .getContext()
                .getItemStateManager(), si.getNamespaceMappings(), si.getTextAnalyzer(), si
                .getContext()
                .getPropertyTypeRegistry(), si.getSynonymProvider(), si.getIndexFormatVersion());

            Query luceneQuery = new BooleanQuery();
            return luceneQuery.combine(new Query[]{queryChild, querySelf });

        }
        catch (InvalidQueryException e)
        {
            util.log.error("Invalid query", e);
        }
        catch (RepositoryException e)
        {
            util.log.error("Repository Exception", e);
        }
        return null;
    }
}
